@@ -1,7 +1,7 @@
 /*
  *  smbmount.c
  *
- *  Copyright (C) 1995 by Paal-Kr. Engstad and Volker Lendecke
+ *  Copyright (C) 1995, 1996 by Paal-Kr. Engstad and Volker Lendecke
  *
  */
 
@@ -55,13 +55,16 @@ help(void)
 {
         printf("\n");
         printf("usage: %s //server/service mount-point [options]\n", progname);
-        printf("\n"
+        printf("Version %s\n"
+	       "\n"
                "-p port        Port to connect to (used only for testing)\n"
                "-m max_xmit    max_xmit offered (used only for testing)\n"
                "\n"
                "-s servername  Netbios name of server\n"
                "-c clientname  Netbios name of client\n"
+               "-I machinename The hostname of the machine\n"
                "-U username    Username sent to server\n"
+	       "-D domain      Domain name\n"
                "-u uid         uid the mounted files get\n"
                "-g gid         gid the mounted files get\n"
                "-f mode        permission the files get (octal notation)\n"
@@ -72,12 +75,13 @@ help(void)
                "               If neither -P nor -n are given, you are\n"
                "               asked for a password.\n"
                "-h             print this help text\n"
-               "\n");
+               "\n", VERSION);
 }
 
 static int
 parse_args(int argc, char *argv[], struct smb_mount_data *data,
-           int *got_password, int *upcase_password)
+           int *got_password, int *upcase_password,
+	   char *realhost, int realhost_len)
 {
         int opt;
         struct passwd *pwd;
@@ -86,8 +90,10 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
         *got_password = 0;
         *upcase_password = 1;
 
-        while ((opt = getopt (argc, argv, "Cp:s:c:U:u:g:f:d:m:P:n")) != EOF) {
-                switch (opt) {
+        while ((opt = getopt (argc, argv, "Cp:s:c:U:D:u:g:f:d:m:P:nI:")) != EOF)
+	{
+                switch (opt)
+		{
                 case 'C':
                         *upcase_password = 0;
                         break;
@@ -95,7 +101,8 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
                         data->addr.sin_port = htons(atoi(optarg));
                         break;
                 case 's':
-                        if (strlen(optarg) > 16) {
+                        if (strlen(optarg) > sizeof(data->server_name)-1)
+			{
                                 fprintf(stderr, "Server name too long: %s\n",
                                         optarg);
                                 return 1;
@@ -104,7 +111,8 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
                         strcpy(data->server_name, optarg);
                         break;
                 case 'c':
-                        if (strlen(optarg) > 16) {
+                        if (strlen(optarg) > sizeof(data->client_name)-1)
+			{
                                 fprintf(stderr, "Client name too long: %s\n",
                                         optarg);
                                 return 1;
@@ -112,19 +120,41 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
                         strcpy(data->client_name, optarg);
                         break;
                 case 'U':
-                        if (strlen(optarg) > 63) {
+                        if (strlen(optarg) > sizeof(data->username)-1)
+			{
                                 fprintf(stderr, "Username too long: %s\n",
                                         optarg);
                                 return 1;
                         }
                         strcpy(data->username, optarg);
                         break;
+		case 'D':
+#if SMB_MOUNT_VERSION < 5
+			fprintf(stderr, "The kernel you compiled the smbfs "
+				"utils with is not able\nto accept a domain "
+				"name.\nPlease upgrade your kernel AND "
+				"recompile the smbfs utils\n\n");
+			return 1;
+#else
+			if (strlen(optarg) > sizeof(data->domain)-1)
+			{
+                                fprintf(stderr, "Domain name too long: %s\n",
+                                        optarg);
+                                return 1;
+                        }
+                        strcpy(data->domain, optarg);
+                        break;
+#endif
                 case 'u':
-                        if (isdigit(optarg[0])) {
+                        if (isdigit(optarg[0]))
+			{
                                 data->uid = atoi(optarg);
-                        } else {
+                        }
+			else
+			{
                                 pwd = getpwnam(optarg);
-                                if (pwd == NULL) {
+                                if (pwd == NULL)
+				{
                                         fprintf(stderr, "Unknown user: %s\n",
                                                 optarg);
                                         return 1;
@@ -133,11 +163,15 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
                         }
                         break;
                 case 'g':
-                        if (isdigit(optarg[0])) {
+                        if (isdigit(optarg[0]))
+			{
                                 data->gid = atoi(optarg);
-                        } else {
+                        }
+			else
+			{
                                 grp = getgrnam(optarg);
-                                if (grp == NULL) {
+                                if (grp == NULL)
+				{
                                         fprintf(stderr, "Unknown group: %s\n",
                                                 optarg);
                                         return 1;
@@ -155,12 +189,26 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
                         data->max_xmit = atoi(optarg);
                         break;
                 case 'P':
+			if (strlen(optarg) > sizeof(data->password)-1)
+			{
+				fprintf(stderr, "Password too long\n");
+				return 1;
+			}
                         strcpy(data->password, optarg);
                         *got_password = 1;
                         break;
                 case 'n':
                         *got_password = 1;
                         break;
+                case 'I':
+			if (strlen(optarg) > realhost_len-1)
+			{
+				fprintf(stderr, "Hostname too long: %s\n",
+					optarg);
+				return 1;
+			}
+                        strcpy(realhost, optarg);
+                        break;
                 default:
                         return -1;
                 }
@@ -169,20 +217,6 @@ parse_args(int argc, char *argv[], struct smb_mount_data *data,
         
 }
 
-#if 0
-static char **
-parse_optfile(const char *user, const char *server, const char *service)
-{
-        char filename[1024];
-        char complete_service[1024];
-
-        static char line[1024];
-        static char *argv[50];
-
-        return argv;
-}
-#endif
-
 static int
 extract_service(const char *service, char **server, char **share, char **root,
                 char **user)
@@ -239,7 +273,7 @@ extract_service(const char *service, char **server, char **share, char **root,
 
                 int i;
 
-                if (strlen(root_start) > 62) {
+                if (strlen(root_start) > sizeof(s_root)-2) {
                         fprintf(stderr, "root too long: %s\n", root_start);
                         return -1;
                 }
@@ -264,6 +298,12 @@ extract_service(const char *service, char **server, char **share, char **root,
         user_start = strchr(share_start, '%');
 
         if (user_start != NULL) {
+		if (strlen(user_start+1) > sizeof(s_user)-1)
+		{
+			fprintf(stderr, "user too long: %s\n", user_start+1);
+			return -1;
+		}
+
                 user_start[0] = '\0';
                 user_start += 1;
                 strcpy(s_user, user_start);
@@ -271,6 +311,7 @@ extract_service(const char *service, char **server, char **share, char **root,
                 s_user[0] = '\0';
         }
 
+	/* The length of these has already been checked */
         strcpy(s_server, complete_service);
         strcpy(s_share,  share_start);
 
@@ -286,79 +327,79 @@ fullpath(const char *p)
 {
         char path[MAXPATHLEN];
 
+	if (strlen(p) > MAXPATHLEN-1)
+	{
+		return NULL;
+	}
+
         if (realpath(p, path) == NULL)
+	{
                 return strdup(p);
-        else
-                return strdup(path);
+	}
+	return strdup(path);
 }
 
+#ifndef HAVE_KERNELD
+
 /* Returns 0 if the filesystem is in the kernel after this routine
    completes */
 static int
-load_smbfs()
+load_smbfs(void)
 {
-	FILE *fver, *ffs;
+	FILE *ffs;
 	char s[1024];
-	char modname[1024];
 	char *p, *p1;
         pid_t pid;
         int status;
 
 	/* Check if smbfs is in the kernel */
 	ffs = fopen("/proc/filesystems", "r");
-	if (ffs == NULL) {
-		perror("Error: \"/proc/filesystems\" could not be read:");
+
+	if (ffs == NULL)
+	{
+		perror("Error: \"/proc/filesystems\" could not be read");
 		return -1;
 	}
+
 	p = NULL;
-	while (! feof(ffs)) {
+	while (! feof(ffs))
+	{
 		p1 = fgets(s, sizeof(s), ffs);
-		if (p1) {
+		if (p1)
+		{
 			p = strstr(s, "smbfs");
-			if (p) {
+			if (p)
+			{
 				break;
 			}
 		}
 	}
 	fclose(ffs);
-	if (p) {
+
+	if (p)
+	{
 		return 0;
 	}
-	fver = fopen("/proc/version", "r");
-	if (fver == NULL) {
-		perror("Error: \"/proc/version\" could not be read:");
-		return -1;
-	}
-	fgets(s, 1024, fver);
-	fclose(fver);
-	p = strstr(s, "version ");
-	if (p == NULL) {
-	version_error:
-		fprintf(stderr, "Error: Unable to determine the Linux version"
-                        "from \"/proc/version\n");
-		return -1;
-	}
-        p = strchr(p, ' ') + 1;
-	p1 = strchr(p, ' ');
-	if (p1 == NULL)  {
-		goto version_error;
-	}
-        strcpy(modname, "/lib/modules/");
-        strncat(modname, p, p1 - p);
-        strcat(modname, "/smbfs.o");
 
         /* system() function without signal handling, from Stevens */
 
-        if ((pid = fork()) < 0) {
+        if ((pid = fork()) < 0)
+	{
                 return 1;
-        } else if (pid == 0) {
+        }
+	else if (pid == 0)
+	{
                 /* child */
-                execl("/sbin/insmod", "insmod", modname, NULL);
+                execl("/sbin/modprobe", "modprobe", "smbfs", NULL);
                 _exit(127);     /* execl error */
-        } else {
+        }
+	else
+	{
                 /* parent */
-                while (waitpid(pid, &status, 0) < 0) {
-                        if (errno != EINTR) {
+                while (waitpid(pid, &status, 0) < 0)
+		{
+                        if (errno != EINTR)
+			{
                                 status = -1;
                                 break;
                         }
@@ -367,6 +408,8 @@ load_smbfs()
         return status;
 }
 
+#endif /* HAVE_KERNELD */
+
 /* Check whether user is allowed to mount on the specified mount point */
 static int
 mount_ok(struct stat *st)
@@ -388,8 +431,6 @@ mount_ok(struct stat *st)
         return 0;
 }
 
-
-
 int 
 main(int argc, char *argv[])
 {
@@ -402,6 +443,7 @@ main(int argc, char *argv[])
         int um;
 	unsigned int flags;
 	char hostname[MAXHOSTNAMELEN + 1];
+	char realhost[MAXHOSTNAMELEN + 1];
 
 	struct hostent *h;
         char *server;
@@ -417,11 +459,6 @@ main(int argc, char *argv[])
 
         progname = argv[0];
 
-        if (geteuid() != 0) {
-                fprintf(stderr, "%s must be installed suid root\n", progname);
-                exit(1);
-        }
-
 	memset(&data, 0, sizeof(struct smb_mount_data));
 
 	memset(hostname, '\0', MAXHOSTNAMELEN+1);
@@ -443,6 +480,11 @@ main(int argc, char *argv[])
                 }
         }
 
+        if (geteuid() != 0) {
+                fprintf(stderr, "%s must be installed suid root\n", progname);
+                exit(1);
+        }
+
         service = argv[1];
         mount_point = argv[2];
 
@@ -461,39 +503,26 @@ main(int argc, char *argv[])
         }
 
         if (mount_ok(&st) != 0) {
-                fprintf(stderr, "cannot to mount on %s: %s\n",
+                fprintf(stderr, "cannot mount on %s: %s\n",
                         mount_point, strerror(errno));
                 exit(1);
         }
 
-        if ((h = gethostbyname(server)) == NULL) {
-		printf("%s: unknown host\n", server);
-		return 0;
-	}
-
+#ifndef HAVE_KERNELD
 	/* Check if the smbfs filesystem is in the kernel.  If not, attempt
 	 * to load the smbfs module */
 	if (load_smbfs() != 0) {
 		fprintf(stderr, "Error: Unable to start smbfs, exiting...\n");
 		exit(1);
 	}
+#endif
 
-	data.addr.sin_family      = AF_INET;
-        data.addr.sin_addr.s_addr = ((struct in_addr *)(h->h_addr))->s_addr;
-	data.addr.sin_port        = htons(SMB_PORT);
-	
-	data.fd = socket(AF_INET, SOCK_STREAM, 0);
-	if (data.fd == -1) {
-		perror("socket");
-                return -1;
-	}
-	
 	data.version = SMB_MOUNT_VERSION;
 
         /* getuid() gives us the real uid, who may umount the fs */
         data.mounted_uid = getuid();
 
-        strcpy(data.service, share);
+	sprintf(data.service, "\\\\%s\\%s", server, share);
         str_upper(data.service);
 
         strcpy(data.root_path, root);
@@ -516,13 +545,42 @@ main(int argc, char *argv[])
         umask(um);
         data.file_mode = (S_IRWXU|S_IRWXG|S_IRWXO) & ~um;
         data.dir_mode  = 0;
+	data.addr.sin_family = AF_INET;
+	data.addr.sin_port = htons(SMB_PORT);
+
+	strcpy(data.domain, "?");
+
+        realhost[0] = '\0';
 
         if (parse_args(argc, argv, &data, &Got_Password,
-                       &Upcase_Password) != 0) {
+                       &Upcase_Password, realhost, sizeof(realhost)) != 0) {
                 usage();
                 return -1;
         }
 
+        if (realhost[0] == '\0') {
+		if ((h = gethostbyname(server)) == NULL) {
+			printf("%s: unknown host\n", server);
+			printf("\tThe -I option may be useful.\n");
+			return 0;
+		}
+	}
+	else {
+		if ((h = gethostbyname(realhost)) == NULL) {
+			printf("%s: unknown host\n", realhost);
+			return 0;
+		}
+	}
+	
+        data.addr.sin_addr.s_addr = ((struct in_addr *)(h->h_addr))->s_addr;
+	
+	data.fd = socket(AF_INET, SOCK_STREAM, 0);
+	if (data.fd == -1) {
+		perror("socket");
+                return -1;
+	}
+	
+
         if (data.dir_mode == 0) {
                 data.dir_mode = data.file_mode;
                 if ((data.dir_mode & S_IRUSR) != 0)
@@ -534,15 +592,24 @@ main(int argc, char *argv[])
         }
 
         if (Got_Password == 0) {
-                strcpy(data.password, getpass("Password: "));
+		char *pw = getpass("Password: ");
+		if (strlen(pw) > sizeof(data.password)-1)
+		{
+			fprintf(stderr, "Password too long\n");
+			return -1;
+		}
+                strcpy(data.password, pw);
         }
 
-        if (Upcase_Password == 1) {
+        if (Upcase_Password == 1)
+	{
                 str_upper(data.password);
         }
 
-        if (data.server_name[0] == '\0') {
-                if (strlen(server) > 16) {
+        if (data.server_name[0] == '\0')
+	{
+                if (strlen(server) > sizeof(data.server_name)-1)
+		{
                         fprintf(stderr, "server name too long as a netbios "
                                 "name: %s\n", server);
                         fprintf(stderr, "use option -s server_name\n");
@@ -553,8 +620,10 @@ main(int argc, char *argv[])
         }
 
         
-        if (data.client_name[0] == '\0') {
-                if (strlen(hostname) > 16) {
+        if (data.client_name[0] == '\0')
+	{
+                if (strlen(hostname) > sizeof(data.client_name)-1)
+		{
                         fprintf(stderr, "my hostname name too long as a "
                                 "netbios name: %s\n", hostname);
                         fprintf(stderr, "use option -c client_name\n");
@@ -570,8 +639,8 @@ main(int argc, char *argv[])
                   flags, (char *)&data) < 0) {
 		perror("mount error");
 	        close(data.fd);
-                printf("Maybe you should try to type the username and\n"
-                       "password in UPPERCASE.\n");
+		printf("Please look at smbmount's manual page for "
+		       "possible reasons\n");
 		return -1;
 	}
 
@@ -585,6 +654,12 @@ main(int argc, char *argv[])
         ment.mnt_passno= 0;
 
         mount_point = ment.mnt_dir;
+
+	if (mount_point == NULL)
+	{
+		fprintf(stderr, "Mount point too long\n");
+		return -1;
+	}
 	
         if ((fd = open(MOUNTED"~", O_RDWR|O_CREAT|O_EXCL, 0600)) == -1)
         {
